#include "logMap.h"

void computeKrawczykOperator(
    LogMap& f,
    const Interval* center,
    const Interval* X,
    const Interval* A,
    const Interval* fcenter,
    Interval* o_periodicTrajectory,
    unsigned long period,
    int threadNumber
   )
{
  static Interval da[numberOfProcessors*maximumPeriod],
                  dsize[numberOfProcessors*maximumPeriod];
  Interval *a = da + threadNumber*maximumPeriod,
           *size = dsize + threadNumber*maximumPeriod;
  
  // we are searching for the possition on which derivative contains zero
  int shift = 0, i, j;
  while(! isSingular(A[shift]) && shift<period)
    ++shift;
  if(shift==period)
    throw "Interval Krawczyk operator, shift=period!!!\n";
  if(! X[shift].contains(LogMap::criticalPoint()))
    throw "Interval Krawczyk operator, X[shift] does not contain critical point!!1\n";

  for(i=0;i<period;++i)
  {
    a[i] = A[i].mid();
    o_periodicTrajectory[i] = Real(0.);
    size[i]= X[i]-center[i];
  }
  // we force a[shift]=0
  a[shift] = Real(0.);

  // computing (Id - C*DF)*(X-center)
  // the procedure computes coefficients of columns of matrix (Id - C*DF)
  // after that a part of the sum in product (Id - C*DF)*(X-center)
  // is stored in o_periodicTrajectory.
  // This saves the memory usage - we do not need to store at all the matrix
  
  for(i=0;i<period;++i)
  {
    int indeks = (shift+i)%period;
    Interval product = A[indeks] - a[indeks];
    o_periodicTrajectory[(indeks+1)%period] += product*size[indeks];
    for(j=i+1;j<period;++j)
    {
      indeks = (shift+j)%period;
      product *= a[indeks];
      o_periodicTrajectory[(indeks+1)%period] += product*size[indeks];      
    }
  }
  
  // at the and we must must add C* F(center), 
  Interval u = fcenter[shift%period];
  o_periodicTrajectory[(shift+1)%period] = u - o_periodicTrajectory[(shift+1)%period];
  for(i=1;i<period;++i)
  {
    int indeks = (shift+i)%period;
    int indeks2 = (shift+i+1)%period;
    u = fcenter[indeks] + a[indeks]*u;
    o_periodicTrajectory[indeks2] = u - o_periodicTrajectory[indeks2];
  }
}


