#include <memory>
#include "capd/vectalg/Vector.hpp"

#ifndef _PARAMETER_INFO_H_
#define _PARAMETER_INFO_H_

enum ParameterFlag{unknown,stability,uncheckable,convergenceWithoutProof,existenceWithoutStability,periodDoubling};

template<class IntervalT>
struct ParameterInfo
{
  typedef IntervalT IntervalType;
  typedef typename IntervalT::BoundType FloatType;
  typedef capd::vectalg::Vector<IntervalType,0> IVector;

  ParameterInfo() : c(0), flag(unknown), period(0), enclosure(0){}
  
  ParameterInfo(const IntervalType& _c) 
    : c(_c), flag(unknown), period(0), enclosure(0)
  {}
  
  ParameterInfo(const IntervalType& _c, ParameterFlag _flag, IntervalType* data, unsigned int period) 
    : c(_c), flag(_flag), period(period), enclosure( new IVector(period,data) )
  {}
  
  friend bool operator < (const ParameterInfo& a, const ParameterInfo& b)
  {
    return a.c.leftBound() < b.c.leftBound();
  }
  
  IntervalType c;
  ParameterFlag flag;
  unsigned int period;
  IVector* enclosure;
};

template<class IntervalT>
std::istream& operator >> (std::istream& in, ParameterInfo<IntervalT>& p)
{
  typedef typename ParameterInfo<IntervalT>::IVector IVector;
  in >> p.c;
  int i;
  in >> i;
  p.flag = ParameterFlag(i);
  if(p.flag==stability || p.flag==periodDoubling)
  {
    in >> p.period;
    if(!(p.enclosure))
    {
      p.enclosure = new IVector(1);
    }
    in >> *(p.enclosure);
  }
  return in;
}

template<class IntervalT>
std::ostream& operator << (std::ostream& out, const ParameterInfo<IntervalT>& p)
{
  out << p.c << " " << p.flag;
  if(p.flag== stability || p.flag==periodDoubling){
    out << " " << p.enclosure->dimension()<< " " << *(p.enclosure) << " ";
  }
  return out;
}

#endif
