#ifndef __PARAMETER_READER_H__
#define __PARAMETER_READER_H__

#include <string>
#include <sstream>
#include <map>
#include <iostream>
#include <fstream>


class ParameterReader
{
public:
  typedef std::string Key;
  typedef std::string Value;
  typedef std::map<Key,Value> ArgTable;
  typedef ArgTable::iterator iterator;

  // this function clears the table of parameters
  void clear(){
    m_args.clear();
  }

  // this function finds a key and returns a value
  // if there is no key in hash table an empty string is returned
  Value find(Key a_key){
    iterator it = m_args.find(a_key);
    if(it!=m_args.end())
      return it->second;
    else
      return "";
  }

  // this function inserts new value to a table
  void insert(Key a_key, Value a_value){
    m_args[a_key] = a_value;
  }

  void readFromFile(const char* fileName){
    std::ifstream in;
    in.open(fileName);

    // the main loop
    std::string line;
    while( in.good() )
    {
      std::getline(in,line);
      if (line[0] == '@')
      {
        line.erase(0,1);
        std::istringstream inputString(line);
        Key key;
        Value value;
        inputString >> key;
        std::getline(inputString,value);
        if(value.size()>1)
          value.erase(0,1);
        insert(key,value);
      }  
    }

    in.close();
  }

private:
  ArgTable m_args;
};

template<class Data>
Data parseString(const std::string& a_string){
  std::istringstream input(a_string);
  Data result;
  input >> result;
  return result;
}


#endif

