/// @addtogroup auxil
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file commandLineArgs.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#if !defined(_COMMANDLINEARGS_H_)
#define _COMMANDLINEARGS_H_
#include <map>

// Read the arguments in argv, decompose pairs key=value and store
// the (key,value) pairs in the map m provided
// When no = is present treat it as a key with value "1"
void decomposeArgv(int argc,char** argv,std::map<std::string,std::string>& A_m){
  for(int i=1;i<argc;++i){
    std::string arg(argv[i]);
    std::string::size_type idx;
    idx=arg.find("=");
    if(idx == std::string::npos){
      A_m.insert(make_pair(arg,std::string("1")));
    }else{
      std::string argName(arg,0,idx);
      std::string argVal(arg,idx+1,arg.length()-idx-1);
      A_m.insert(make_pair(argName,argVal));
    }
  }
}

// Count the number of times A_val appers as value in the map A_m
// Return the pair (count,arg), where count is the number computed and arg is the key of the first
// value found
std::pair<int,std::string> valCount(const std::string& A_val,std::map<std::string,std::string>& A_m){
  std::map<std::string,std::string>::iterator it;
  int count=0;
  std::string arg;
  for(it=A_m.begin();it!=A_m.end();++it){
    if(it->second == A_val){
      ++count;
      arg=it->first;
    }
  }
  return std::make_pair(count,arg);
}

// A macro declaring an std::map _command_line_args_ for command line args and calling decomposeArgv to setup its values
#define setupCommandLineArgs std::map<std::string,std::string> _command_line_args_; decomposeArgv(argc,argv,_command_line_args_);

// A macro to declare a variable for a named command line argument, assign it a default value and replace it by the command line argument value if provided
#define declareCommandLineArg(type,name,value) type name(value); if(_command_line_args_.count(#name)){std::istringstream iss(_command_line_args_[#name]); iss >> name;}

// A macro to assign a value to a global variable from the command line argument value if provided. The declaration must be provided seperately
#define assignCommandLineArgValue(name) if(_command_line_args_.count(#name)){std::istringstream iss(_command_line_args_[#name]); iss >> name;}

// This is a shortcut for valCOunt above to work on _command_line_args_
#define getValueCountAndArg(value) valCount(value,_command_line_args_);

#endif //_COMMANDLINEARGS_H_

/// @}
