/////////////////////////////////////////////////////////////////////////////
//   Package:          CAPD
//   Subpackage: 
/// @addtogroup intervals
/// @{ 

/////////////////////////////////////////////////////////////////////////////
/// @file mpIntervalFun.h
///
/// @author Tomasz Kapela   @date 23-08-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006 
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 


#ifndef _CAPD_INTERVAL_MPINTERVALFUN_H_ 
#define _CAPD_INTERVAL_MPINTERVALFUN_H_ 

#include <exception>
#include "capd/multiPrec/MpfrClass.h"
#include "capd/interval/MpIntervalSettings.h"

class NaNException: public std::exception{
  const char * what() const throw ()
  {
    return "Not a number. ";
  }
};


#ifdef __HAVE_MPFR__

namespace capd{
namespace intervals{

typedef capd::multiPrec::MpfrClass MpfrClass;


inline bool isinf(const Interval<MpfrClass> & r) 
{ 
  return (isinf(r.leftBound()) || isinf(r.rightBound())); 
}

inline void testNaN(const Interval<MpfrClass> & x)
{
#ifdef __MPI_TEST_NAN__
  if(isnan(x.leftBound()) || isnan(x.rightBound())))
    throw NaNException();  
#endif  // __MPI_TEST_NAN__
}


template<>
Interval<MpfrClass> sin(const Interval<MpfrClass>& x);

template<> 
Interval<MpfrClass> cos(const Interval<MpfrClass>& x);

template<>
Interval<MpfrClass> tan(const Interval<MpfrClass>& x);


template<>
Interval<MpfrClass> log(const Interval<MpfrClass>& x);

template<>
Interval<MpfrClass> exp(const Interval<MpfrClass>& x);

//-------- sqrt -----------------
template<>
Interval<MpfrClass> sqrt(const Interval<MpfrClass>  & z);

template<>
inline Interval<MpfrClass> power(const Interval<MpfrClass> &a, const Interval<MpfrClass> &b)
{
  if( a.leftBound() < 0 )
  {
    throw std::range_error("power(A, B): interval A must be greater than zero\n");
  }
  return exp(b * log(a));
}

template<>
Interval<MpfrClass> power(const Interval<MpfrClass>& value, /*long*/  int exponent);

template<>
inline Interval<MpfrClass>  Interval<MpfrClass>::pi()
{
  
  capd::multiPrec::PrecisionType prec = BoundType::getDefaultPrecision();
  return Interval<MpfrClass>(BoundType::Pi(capd::multiPrec::RoundDown, prec), BoundType::Pi(capd::multiPrec::RoundUp, prec));
}

/*
//  So far not defined in Interval template 
//
template<>
Interval<MpfrClass> Interval<MpfrClass>::log2()
{

   capd::multiPrec::PrecisionType   prec = BoundType::getDefaultPrecision();
   return Interval<MpfrClass>(BoundType::Log2(capd::multiPrec::RoundDown, prec), BoundType::Log2(capd::multiPrec::RoundUp, prec));
}
*/

template<>
inline Interval<MpfrClass> Interval<MpfrClass>::euler()
{
   capd::multiPrec::PrecisionType   prec = BoundType::getDefaultPrecision();
   return Interval<MpfrClass>(BoundType::Euler(capd::multiPrec::RoundDown, prec), BoundType::Euler(capd::multiPrec::RoundUp, prec));
}


}}  // end of namespace capd::intervals

#endif  // __HAVE_MPFR__

#endif // _CAPD_INTERVAL_MPINTERVALFUN_H_ 

/// @}

