#include <iostream>
#include <sstream>
#include "capd/auxil/commandLineArgs.h"
#include "logMap.h"

using namespace std;

void readConfigFile()
{
  ParameterReader r;
  r.readFromFile("configFile");
  
  numberOfSkippedPoints = parseString<unsigned long>(r.find("numberOfSkippedPoints"));
  accuracyForNonrigorousPeriodicOrbit = parseString<Real>(r.find("accuracyForNonrigorousPeriodicOrbit"));

  minimalSizeOfParameterWhenProvingStableOrbit = parseString<Real>(r.find("minimalSizeOfParameterWhenProvingStableOrbit"));
  minimalSizeOfParameterWhenExtendingStableOrbit = parseString<Real>(r.find("minimalSizeOfParameterWhenExtendingStableOrbit"));
  minimalSizeOfParameterWhenApproachPeriodDoubling = parseString<Real>(r.find("minimalSizeOfParameterWhenApproachPeriodDoubling"));
  sizeOfSetForTheNewtonOperator = parseString<Real>(r.find("sizeOfSetForTheNewtonOperator"));
  sizeOfSetForTheNewtonOperatorWhenFillGaps = parseString<Real>(r.find("sizeOfSetForTheNewtonOperatorWhenFillGaps"));
  sizeOfSetForTheNewtonOperatorWhenApproachPeriodDoubling  = parseString<Real>(r.find("sizeOfSetForTheNewtonOperatorWhenApproachPeriodDoubling"));
  sizeOfSetForTheNewtonOperatorWhenProvingStableOrbitForMinimalParameterDiam = parseString<Real>(r.find("sizeOfSetForTheNewtonOperatorWhenProvingStableOrbitForMinimalParameterDiam"));

  cout << "numberOfSkippedPoints " << numberOfSkippedPoints << endl;
  cout << "accuracyForNonrigorousPeriodicOrbit " << accuracyForNonrigorousPeriodicOrbit << endl;

  cout << "minimalSizeOfParameterWhenProvingStableOrbit " << minimalSizeOfParameterWhenProvingStableOrbit << endl;
  cout << "minimalSizeOfParameterWhenExtendingStableOrbit " << minimalSizeOfParameterWhenExtendingStableOrbit << endl;
  cout << "minimalSizeOfParameterWhenApproachPeriodDoubling " << minimalSizeOfParameterWhenApproachPeriodDoubling << endl;
  cout << "sizeOfSetForTheNewtonOperator " << sizeOfSetForTheNewtonOperator << endl;
  cout << "sizeOfSetForTheNewtonOperatorWhenFillGaps " << sizeOfSetForTheNewtonOperatorWhenFillGaps << endl;
  cout << "sizeOfSetForTheNewtonOperatorWhenApproachPeriodDoubling " << sizeOfSetForTheNewtonOperatorWhenApproachPeriodDoubling << endl;
  cout << "sizeOfSetForTheNewtonOperatorWhenProvingStableOrbitForMinimalParameterDiam " << sizeOfSetForTheNewtonOperatorWhenProvingStableOrbitForMinimalParameterDiam << endl;
}

// ************************** MAIN *******************************

int main(int argc, char*argv[])
{
  #ifdef __USE_GNU_MP__
    Real::setDefaultPrecision(__USE_GNU_MP__);
  #endif
  #ifdef __USE_PARALLEL__
    omp_set_num_threads ( capd::min( omp_get_num_procs(), numberOfProcessors ) );
 //   omp_set_num_threads ( 1 );
  #endif

  readConfigFile();
  cout.precision(21);
  LogMap logMap[numberOfProcessors];
  
  // decompose command line arguments
  typedef map<string,string> ArgTable;
  ArgTable arg;
  decomposeArgv(argc,argv,arg);

  ArgTable::iterator input = arg.find("input");
  ArgTable::iterator output = arg.find("output");
  ArgTable::iterator depth = arg.find("depth");
  ArgTable::iterator chunk = arg.find("chunkSize");
  
  int depthOfSubdivision = 1;
  unsigned long chunkSize = 0;
  if(depth!=arg.end())
    depthOfSubdivision = atoi(depth->second.c_str());
  if(chunk!=arg.end())
    chunkSize  = atoi(chunk->second.c_str());

  try{
    int numberOfThreads = capd::min( omp_get_num_procs(), numberOfProcessors);
    DataContainer c;
    cout << "number of threads: " << numberOfThreads << endl << flush;

    // if no input file is given as a parameter then we start to subdivide range [2,4]
    string inFileName="", outFileName;
    if(input==arg.end())
    {
      createFirstGrid(c,100,Interval(2,4));
    }
    else
      inFileName = input->second;
    
    for(int i=0;i<depthOfSubdivision;++i)
    {
      long T = time(0);
      unsigned long totalLines=0, lines;
      ostringstream s;
      if(output!=arg.end())
        s << output->second;
      else
        s << "output/subdivision";
      if(depthOfSubdivision>1)
        s << (i+1);
      
      outFileName = s.str();
      do{
        if(inFileName!="")
        {
          releaseMemory(c);
          c.clear();
          lines = readDataFromFile(inFileName.c_str(),c,totalLines,chunkSize);
            if (!lines) break;
        }
        
        cout << "finding superstable ... " << flush;
        findSuperstable(logMap,c);
        cout << " done" << std::endl;
        cout << "filling gaps";
        fillGaps(logMap,c);
        collapseData(c);
        
        cout << "\nsubdividing container, number of elements = " << c.size() << endl;
        subdivideContainer(logMap,c,numberOfThreads);
        cout << "filling gaps";
        fillGaps(logMap,c);
        collapseData(c);

        cout << "scan for period doubling";
        scanForPeriodDoubling(logMap,c);

        saveDataContainer(outFileName.c_str(),c,!(chunkSize && totalLines));

        totalLines += lines;
      }while(chunkSize && lines == chunkSize); 
      
      inFileName = outFileName;

      cout <<(i+1) << ". ";
      computeTotalWidth(outFileName.c_str());
      std::cout << "time of computations (sec) = " << time(0) - T << std::endl;
    } // for depthOfSubdivision
    releaseMemory(c);

  }catch(const char* s)
  {
    cout << s << endl;
  }
  catch(char* s)
  {
    cout << s << endl;
  }  
  catch(runtime_error& e)
  {
    cout << e.what();
  }  
}

